// Interrupt serial event
void reverseDir(boolean rev){
    stepper.setPinsInverted(rev, false, false);  //for FSQ85(true, false, false) ---  for TSA 120should be (false, false, false)  (dir, step, enable) 
    //not reversed is std setup for tsa-120
    reversed = rev;
}
void Hold(boolean conthold){   
  hold = conthold;
}

void serialEvent(){
  while (Serial.available() > 0) {
    char inChar = (char)Serial.read();    
    if (inChar == ':' || inChar == '#'){
        if(inputString == "PV") inputString = "V"; // Version
        if(inputString == "GA") inputString = "B"; // bit 1 for switch 2
        if(inputString == "SH") inputString = "H"; // Starts the Find Home routine
        if(inputString == "GS") inputString = "Z"; // bit 2 for In limit switch
        if(inputString == "GT") inputString = "D"; // Sensor temperature in tenths of a degree, ex: 25°C = 250d
        if(inputString == "GV") inputString = "E"; // System voltage in tenths of a volt, ex 12.0V = 120d
        if(inputString == "PD") inputString = "D"; // Set the display brightness
        if(inputString == "PL") inputString = "D"; // Set the display sleep brightness
        if(inputString == "PF") inputString = "F"; // Gets the current focuser type. Ex: "2.5 NC#"
        if(inputString == "PS") inputString = "X"; // Gets the current focuser serial number
        if(inputString == "PU") inputString = "W"; // Gets a user defined field
        if(inputString == "PE") inputString = "C"; // Enables or disables the encoders
        if(inputString == "PR") inputString = "J"; // Issues a reset
        if(inputString == "Pt") inputString = "D"; // Adjusts the temperature offset
        if(inputString == "Pu") inputString = "Y"; // Sets the user text field
        serialCommand(inputString); 
        inputString = "";      
    } 
    else {
      inputString += inChar;
    }
  } 
}

void serialCommand(String command) {
  String param = command.substring(4); 
  String comando = command.substring(0,3);  //:2SN 12345678#  
  if(comando == "2GP") command = "G"; // Current position count, 8 digits, signed, 0 padding
  if(comando == "1GP") command = "K"; // DUMMY Current position 
  if(comando == "3GP") command = "K"; // DUMMY Current position   
  if(comando == "2GM") command = "Q"; // IS MOVING  
  if(comando == "2GN") command = "A"; // New (or Target) position count, 8 digits, signed, 0 padding
  if(comando == "2GR") command = "A"; // Motor step delay in 100 microsecond intervals
  if(comando == "2SQ") command = "S"; // Stops motor
  if(comando == "2SM") command = "A"; // Starts motor to move to the “NEW” focus position
  if(comando == "2SP") command = "P"; // Set the current position, 32 bit value, signed
  if(comando == "2SN") command = "M"; // Move to the New position count, 32 bit value, signed
  if(comando == "1SR") command = "A"; // Set the motor step rate in 100 microsecond intervals.
  if(comando == "2SR") command = "A"; // Set the motor step rate in 100 microsecond intervals.
  if(comando == "3SR") command = "A"; // Set the motor step rate in 100 microsecond intervals.

  //Serial.println(command);

  switch(command.charAt(0)) {
    case 'Q':
      if(moving == true){
        Serial.println("01#");
      } else {
        Serial.println("00#");
      }
    case 'Y':
      Serial.println("RC 8#");
      break;
    case 'K':
      Serial.println("30000#");
      break;
    case 'A':
      Serial.println("#");
      break;
    case 'B':
      Serial.println("00#");
      break;
    case 'Z':
      Serial.println("00#");
      break;
    case 'D':
      Serial.println("105#");      
      break;  
    case 'E':
      Serial.println("123#");      
      break;  
    case 'F':
      Serial.println("3.5 NC#");      
      break;  
    case 'X':
      Serial.println("20200516#");      
      break; 
    case 'W':
      Serial.println("StarPI Rotator#");      
      break;
    case 'G':   
      printCurrentPosition();
      break; 
    case 'P':    
      saveCurrentPos(stringToLong(param));
      break;
    case 'H':    // find home
      if(param = 2) findHome();  
      break;
    case 'M': 
      moveStepper(stringToLong(param), false);    
      break;
    case 'S':
       halt();
       break;
    case 'R':      
      int value;
      value = (stringToNumber(param));
      if (value == 0){
        reverseDir(false);
      } else {
        reverseDir(true);
      } 
      break;    
    case 'C':// continuos hold      
      int value2;
      value2 = (stringToNumber(param));      
      if (value2 == 0){
        Hold(false);
      } else{
        Hold(true);
      }
      Serial.println("#");
      break;
    case 'V':
      Serial.println("1.3#");
      break;
    case 'J': // Reset Arduino myFocuserPro2 controller
      software_Reboot();
      break; 
  }
}

void printCurrentPosition() {
  Serial.print("00"); 
  if(stepper.currentPosition()<100000) Serial.print("0");
  if(stepper.currentPosition()<10000) Serial.print("0");
  if(stepper.currentPosition()<1000) Serial.print("0");
  if(stepper.currentPosition()<100) Serial.print("0");
  if(stepper.currentPosition()<10) Serial.print("0");
  Serial.print(stepper.currentPosition());  
  Serial.println("#");
}

void moveStepper(long newPos, boolean manualMove) {
  Serial.println("#");
  if (findingHome == true){
    stepper.moveTo(newPos);
    return;
  }
      
  if(newPos != stepper.currentPosition()) {    
    if(newPos < minStep || newPos > maxSteps) {
      return;
    } else{
        stepper.moveTo(newPos);       
        positionSaved = false;
        firstPrint = false;
    }    
  }
}

void halt() {
  Serial.println("#");
  stepper.stop();  
}

void saveCurrentPos(long newPos) {
  Serial.println("#");
  if(newPos < minStep) newPos = minStep;
  stepper.setCurrentPosition(newPos);
  moveStepper(newPos, false);
  saveFocuserPos(newPos);
  positionSaved = true;
}

void findHome(){
  Serial.println("#");
  long newPos = maxSteps/4;  
  homeFound = true;
  findingHome = false;
  saveCurrentPos(newPos);    
}
